@@ -29,7 +29,7 @@ def process_response(items):
     return items
 
 
-def get_app(path):
+def get_app(path: str):
     from fastapi import FastAPI, HTTPException
     from goldilox.config import ALLOW_CORS, CORS_ORIGINS, ALLOW_HEADERS, ALLOW_METHODS, ALLOW_CREDENTIALS
     logger = logging.getLogger(__name__)
@@ -78,6 +78,20 @@ def get_app(path):
 
         return process_response(ret)
 
+    @app.post("/invocations", response_model=List)
+    def invocations(data: List[Query]):
+        logger.info("/invocations")
+        data = parse_query(data)
+        if len(data) == 0:
+            raise HTTPException(status_code=400, detail="No data provided")
+        try:
+            ret = get_pipeline().predict(data)
+        except Exception as e:
+            logger.error(e)
+            raise HTTPException(status_code=400, detail=str(
+                f"Issue with invocations, try runing your pipeline locally with 'pipeline.inference(data)' to see what is the problem:\n{str(e)}"))
+        return to_nulls(ret)
+
     @app.get("/variables", response_model=dict)
     def variables():
         logger.info("/variables")
@@ -96,7 +110,7 @@ def get_app(path):
     return app
 
 
-def get_wsgi_application(path):
+def get_wsgi_application(path: str):
     from gunicorn.app.base import BaseApplication
 
     class WSGIApplication(BaseApplication):
@@ -143,5 +157,4 @@ class Server:
     def serve(self, options=None):
         options = self._validate_worker_class(options or self.options)
         WSGIApplication = get_wsgi_application(self.path)
-
         WSGIApplication(self.app, options).run()
